/****************************************************************************

    CPanel Class
	Copyright (C) 2006-2008 Andre Deperrois adeperrois@xflr5.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/



#include <QtCore>
#include "Panel.h"
#include <math.h>

double CPanel::s_VortexPos = 0.25;
double CPanel::s_CtrlPos   = 0.25;
double CPanel::mat[9];
double CPanel::det;
CVector CPanel::LATB, CPanel::TALB;


CPanel::CPanel()
{
	Reset();
}

void CPanel::Reset()
{
	Area   = 0.0;
    m_Gamma = 0.0;
    angularPos = 0.0;

    m_AoA = 0;
    m_AoA_old = 0;

    pTB = NULL;
    pTA = NULL;

    isHub            = false;
    isTip            = false;

	m_Pos         =  MIDSURFACE;

	m_iLA         =  0;
	m_iLB         =  0;
	m_iTA         =  0;
	m_iTB         =  0;

    LiftVector.Set(0,0,0);
    DragVector.Set(0,0,0);
    LiftDragVector.Set(0,0,0);

	memset(lij, 0, sizeof(lij));
}


void CPanel::SetFrame(CVector const &LA, CVector const &LB, CVector const &TA, CVector const &TB)
{
    Old_CtrlPt = CtrlPt;

	LATB.x = TB.x - LA.x;
	LATB.y = TB.y - LA.y;
	LATB.z = TB.z - LA.z;
	TALB.x = LB.x - TA.x;
	TALB.y = LB.y - TA.y;
	TALB.z = LB.z - TA.z;

	Normal = LATB * TALB;
	Area = Normal.VAbs()/2.0;
	Normal.Normalize();

	VA.x = LA.x*(1.0-s_VortexPos)+TA.x*s_VortexPos;
	VA.y = LA.y*(1.0-s_VortexPos)+TA.y*s_VortexPos;
	VA.z = LA.z*(1.0-s_VortexPos)+TA.z*s_VortexPos;

	VB.x = LB.x*(1.0-s_VortexPos)+TB.x*s_VortexPos;
	VB.y = LB.y*(1.0-s_VortexPos)+TB.y*s_VortexPos;
	VB.z = LB.z*(1.0-s_VortexPos)+TB.z*s_VortexPos;

	CollPt.x = (LA.x + LB.x + TA.x + TB.x)/4.0;
	CollPt.y = (LA.y + LB.y + TA.y + TB.y)/4.0;
	CollPt.z = (LA.z + LB.z + TA.z + TB.z)/4.0;

		m.x = (LB.x + TB.x) *0.5 - CollPt.x;
		m.y = (LB.y + TB.y) *0.5 - CollPt.y;
		m.z = (LB.z + TB.z) *0.5 - CollPt.z;

	m.Normalize();

	l.x =  m.y * Normal.z - m.z * Normal.y;
	l.y = -m.x * Normal.z + m.z * Normal.x;
	l.z =  m.x * Normal.y - m.y * Normal.x;

	//create the transformation matrix
	lij[0]=l.x;
	lij[1]=m.x;
	lij[2]=Normal.x;
	lij[3]=l.y;
	lij[4]=m.y;
	lij[5]=Normal.y;
	lij[6]=l.z;
	lij[7]=m.z;
	lij[8]=Normal.z;
	Invert33(lij);

	if(m_Pos>MIDSURFACE)
	{
		P1.x = lij[0]*(LA.x-CollPt.x) + lij[1]*(LA.y-CollPt.y) + lij[2]*(LA.z-CollPt.z);
		P1.y = lij[3]*(LA.x-CollPt.x) + lij[4]*(LA.y-CollPt.y) + lij[5]*(LA.z-CollPt.z);
		P1.z = lij[6]*(LA.x-CollPt.x) + lij[7]*(LA.y-CollPt.y) + lij[8]*(LA.z-CollPt.z);
		P2.x = lij[0]*(LB.x-CollPt.x) + lij[1]*(LB.y-CollPt.y) + lij[2]*(LB.z-CollPt.z);
		P2.y = lij[3]*(LB.x-CollPt.x) + lij[4]*(LB.y-CollPt.y) + lij[5]*(LB.z-CollPt.z);
		P2.z = lij[6]*(LB.x-CollPt.x) + lij[7]*(LB.y-CollPt.y) + lij[8]*(LB.z-CollPt.z);
		P3.x = lij[0]*(TB.x-CollPt.x) + lij[1]*(TB.y-CollPt.y) + lij[2]*(TB.z-CollPt.z);
		P3.y = lij[3]*(TB.x-CollPt.x) + lij[4]*(TB.y-CollPt.y) + lij[5]*(TB.z-CollPt.z);
		P3.z = lij[6]*(TB.x-CollPt.x) + lij[7]*(TB.y-CollPt.y) + lij[8]*(TB.z-CollPt.z);
		P4.x = lij[0]*(TA.x-CollPt.x) + lij[1]*(TA.y-CollPt.y) + lij[2]*(TA.z-CollPt.z);
		P4.y = lij[3]*(TA.x-CollPt.x) + lij[4]*(TA.y-CollPt.y) + lij[5]*(TA.z-CollPt.z);
		P4.z = lij[6]*(TA.x-CollPt.x) + lij[7]*(TA.y-CollPt.y) + lij[8]*(TA.z-CollPt.z);
	}
	else
	{
		P1.x = lij[0]*(LB.x-CollPt.x) + lij[1]*(LB.y-CollPt.y) + lij[2]*(LB.z-CollPt.z);
		P1.y = lij[3]*(LB.x-CollPt.x) + lij[4]*(LB.y-CollPt.y) + lij[5]*(LB.z-CollPt.z);
		P1.z = lij[6]*(LB.x-CollPt.x) + lij[7]*(LB.y-CollPt.y) + lij[8]*(LB.z-CollPt.z);
		P2.x = lij[0]*(LA.x-CollPt.x) + lij[1]*(LA.y-CollPt.y) + lij[2]*(LA.z-CollPt.z);
		P2.y = lij[3]*(LA.x-CollPt.x) + lij[4]*(LA.y-CollPt.y) + lij[5]*(LA.z-CollPt.z);
		P2.z = lij[6]*(LA.x-CollPt.x) + lij[7]*(LA.y-CollPt.y) + lij[8]*(LA.z-CollPt.z);
		P3.x = lij[0]*(TA.x-CollPt.x) + lij[1]*(TA.y-CollPt.y) + lij[2]*(TA.z-CollPt.z);
		P3.y = lij[3]*(TA.x-CollPt.x) + lij[4]*(TA.y-CollPt.y) + lij[5]*(TA.z-CollPt.z);
		P3.z = lij[6]*(TA.x-CollPt.x) + lij[7]*(TA.y-CollPt.y) + lij[8]*(TA.z-CollPt.z);
		P4.x = lij[0]*(TB.x-CollPt.x) + lij[1]*(TB.y-CollPt.y) + lij[2]*(TB.z-CollPt.z);
		P4.y = lij[3]*(TB.x-CollPt.x) + lij[4]*(TB.y-CollPt.y) + lij[5]*(TB.z-CollPt.z);
		P4.z = lij[6]*(TB.x-CollPt.x) + lij[7]*(TB.y-CollPt.y) + lij[8]*(TB.z-CollPt.z);
	}

    //////////////new code TW

    CVector MidL, MidT, MiL, MiT, A, B, lc ;

    VortPtA.x = LA.x*(1.0-s_VortexPos)+TA.x*s_VortexPos;
    VortPtA.y = LA.y*(1.0-s_VortexPos)+TA.y*s_VortexPos;
    VortPtA.z = LA.z*(1.0-s_VortexPos)+TA.z*s_VortexPos;

    VortPtB.x = LB.x*(1.0-s_VortexPos)+TB.x*s_VortexPos;
    VortPtB.y = LB.y*(1.0-s_VortexPos)+TB.y*s_VortexPos;
    VortPtB.z = LB.z*(1.0-s_VortexPos)+TB.z*s_VortexPos;

    A.x = LA.x*(1.0-s_VortexPos)+TA.x*s_VortexPos;
    A.y = LA.y*(1.0-s_VortexPos)+TA.y*s_VortexPos;
    A.z = LA.z*(1.0-s_VortexPos)+TA.z*s_VortexPos;

    B.x = LB.x*(1.0-s_VortexPos)+TB.x*s_VortexPos;
    B.y = LB.y*(1.0-s_VortexPos)+TB.y*s_VortexPos;
    B.z = LB.z*(1.0-s_VortexPos)+TB.z*s_VortexPos;

    MidL.x=LA.x+(LB.x-LA.x)*0.5;
    MidL.y=LA.y+(LB.y-LA.y)*0.5;
    MidL.z=LA.z+(LB.z-LA.z)*0.5;
    MidT.x=TA.x+(TB.x-TA.x)*0.5;
    MidT.y=TA.y+(TB.y-TA.y)*0.5;
    MidT.z=TA.z+(TB.z-TA.z)*0.5;



    //ControlPt.x =
    lc.x=B.x-A.x;
    lc.y=B.y-A.y;
    lc.z=B.z-A.z;

    // MiL und MiT
    MiL.x = MidL.x;
    MiL.y = MidL.y;
    MiL.z = MidL.z;

    MiT.x = MidT.x;
    MiT.y = MidT.y;
    MiT.z = MidT.z;

    a1.x = MiT.x-MiL.x;
    a1.y = MiT.y-MiL.y;
    a1.z = MiT.z-MiL.z;

    a3 = a1 *(lc);
    a2 = a1 * a3;
    a1.Normalize();
    a3.Normalize();
    a2.Normalize();
    ///end new code TW

    CtrlPt.x = (MidL.x + (MidT.x - MidL.x)*s_CtrlPos);
    CtrlPt.y = (MidL.y + (MidT.y - MidL.y)*s_CtrlPos);
    CtrlPt.z = (MidL.z + (MidT.z - MidL.z)*s_CtrlPos);

    if (pTB && pTA){
    LALB = CVector(VortPtB - VortPtA).VAbs();
    LBTB = CVector(*pTB - VortPtB).VAbs();
    TALA = CVector(VortPtA - *pTA).VAbs();
    TBTA = CVector(*pTA - *pTB).VAbs();
    }

}



bool CPanel::Invert33(double *l)
{
	memcpy(mat,l,sizeof(mat));
/*		a0 b1 c2
        d3 e4 f5
		g6 h7 i8

	          1                     (ei-fh)   (bi-ch)   (bf-ce)
-----------------------------   x   (fg-di)   (ai-cg)   (cd-af)
a(ei-fh) - b(di-fg) + c(dh-eg)      (dh-eg)   (bg-ah)   (ae-bd)*/

	det  = mat[0] *(mat[4] * mat[8] - mat[5]* mat[7]);
	det -= mat[1] *(mat[3] * mat[8] - mat[5]* mat[6]);
	det += mat[2] *(mat[3] * mat[7] - mat[4]* mat[6]);
	if(det==0.0) return false;
	
	* l     = (mat[4] * mat[8] - mat[5] * mat[7])/det;
	*(l+1)  = (mat[2] * mat[7] - mat[1] * mat[8])/det;
	*(l+2)  = (mat[1] * mat[5] - mat[2] * mat[4])/det;

	*(l+3)  = (mat[5] * mat[6] - mat[3] * mat[8])/det;
	*(l+4)  = (mat[0] * mat[8] - mat[2] * mat[6])/det;
	*(l+5)  = (mat[2] * mat[3] - mat[0] * mat[5])/det;

	*(l+6)  = (mat[3] * mat[7] - mat[4] * mat[6])/det;
	*(l+7)  = (mat[1] * mat[6] - mat[0] * mat[7])/det;
	*(l+8)  = (mat[0] * mat[4] - mat[1] * mat[3])/det;

	return true;
}


double CPanel::GetArea()
{
	return Area;
}



