/****************************************************************************

    CPanel Class
	Copyright (C) 2006 Andre Deperrois adeperrois@xflr5.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/



#ifndef CPANEL_H
#define CPANEL_H


#include "Quaternion.h"
#include "CVector.h"
#include "../XLLT/VortexNode.h"

typedef enum {BOTSURFACE, MIDSURFACE, TOPSURFACE, SIDESURFACE, BODYSURFACE} enumPanelPosition;

class CPanel
{
	friend class CSurface;
    friend class CBlade;
    friend class QLLTSimulation;

public:
	CPanel();

	void Reset();
	void SetFrame(CVector const &LA, CVector const &LB, CVector const &TA, CVector const &TB);
	bool Invert33(double *l);
	double GetArea();

    CVector P1, P2, P3, P4;//point's local coordinates
    CVector m, l;

	double Area;
	double lij[9];

    static CVector LATB, TALB;

	static double s_VortexPos;//between 0 and 1; usually the vortex is positioned at the panel's quarter chord : s_VortexPos=0.25
	static double s_CtrlPos;//between 0 and 1; usually the control point is positioned at the panel's 3/4 chord : s_VortexPos=0.75
	static double det;
	static double mat[9];


    double m_Gamma;             //current panel circulation    
    double m_Gamma_old;         //old panel circulation from last iteration
    double m_Gamma_new;         //new computed panel circulation
    double m_DeltaGamma;        //difference between gamma from last iteration and current value
    double m_Gamma_lastTimestep;

    double m_CL;                //panel lift coefficient
    double m_CD;                //panel drag coefficient
    double m_RE;                //reynolds number at which CL and CD have been obtained

    double LALB,LBTB,TBTA,TALA;

    CVector m_V_total;          //the relative total velocity on the panel in absolute coordinates
    CVector m_V_inPlane;       //the relative total velocity in a1/a3 plane of the panel in absolute coordinates
    CVector m_V_induced;        //the velocity that is induced on the panel from wake and blade vorticity in absolute coordinates
    CVector m_V_sampled;        //rotationally sampled velocity from wind input
    CVector m_V_rotational;     //velocity solely from blade rotation
    CVector m_V_tower;          //velocity induced from tower influence


    CVector VortPtA;
    CVector VortPtB;
    CVector a1,a2,a3;

    double m_AoA, m_AoA_old;               //the angle of attack at the vortex position relative to the panel midline
    bool isHub, isTip;
    int fromTimestep;           //the timestep at which the panel was shed
    int fromStation;            //the blade station at which the panel was shed

    VortexNode* pLA;              //pointer to the LA node
    VortexNode* pLB;              //pointer to the LB node
    VortexNode* pTA;              //pointer to the TA node
    VortexNode* pTB;              //pointer to the TB node

public:
	enumPanelPosition m_Pos; // defines if the panel is positioned on a top, middle, bottom, side or body surface
	int m_iLA, m_iLB, m_iTA, m_iTB;//Corner Node numbers
	CVector Normal; // the unit vector normal to the panel
	CVector CtrlPt; // the control point for VLM analysis or 3D/Thin panels
    CVector Old_CtrlPt; // the control point for VLM analysis or 3D/Thin panels
	CVector CollPt; // the collocation point for 3d panel analysis
	CVector VA, VB; //the left and right end points of the vortex on this panel
    CVector tangentialVector, radialVector;
    CVector LiftVector, DragVector, LiftDragVector;
    double angularPos; //the blade position in degree
    int bladeNumber;
    double twistAngle;
    double chord;
    double length;
    double heigth;
    double radius;


};

#endif
