#ifndef CDMSDATA_H
#define CDMSDATA_H

#include <QList>
#include <QString>
#include <QColor>
#include "DData.h"

class CDMSData : public StorableObject
{
	friend class QBEM;
	friend class QDMS;
    friend class MainFrame;

public:
	static CDMSData* newBySerialize ();
	CDMSData();
	virtual ~CDMSData();
	static QStringList prepareMissingObjectMessage();

public:
	void Compute(DData *pDData, CBlade *pWing, double lambda, double pitch, double windspeed);
    void Serialize(QDataStream &ar, bool bIsStoring, int ArchiveFormat);
	void serialize();  // override from StorableObject
    void initArrays(int wtimes, int rtimes, int ptimes);
    void DeleteArrays();



private:
    QString m_WingName;
    QString m_SimName;

    float*** m_Lambda;
    float*** m_one_over_Lambda;
    float*** m_V;
    float*** m_w;
    float*** m_Pitch;
    float*** m_Cp;
    float*** m_Cp1;
    float*** m_Cp2;
    float*** m_Kp;
    float*** m_Cm;
    float*** m_Cm1;
    float*** m_Cm2;
    float*** m_P;
    float*** m_Torque;

    bool simulated;

    int windtimes;
    int pitchtimes;
    int rottimes;

    double windstart;
    double windend;
    double winddelta;

    double rotstart;
    double rotend;
    double rotdelta;

    double pitchstart;
    double pitchend;
    double pitchdelta;

    bool m_bShowPoints;
    bool m_bIsVisible;
    int m_Style;
    int m_Width;
    QColor m_Color;

	//simulation parameters//
	double rho;
	double elements;
	double epsilon;
	double iterations;
	double relax;
	double visc;
	bool m_bPowerLaw;
	bool m_bConstant;
	bool m_bLogarithmic;
	double exponent;
	double roughness;
	bool m_bTipLoss;
	bool m_bAspectRatio;
	bool m_bVariable;
};

#endif // CDMSDATA_H
