/****************************************************************************

    QLLTDock Class
        Copyright (C) 2015 David Marten david.marten@tu-berlin.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#include "QLLTDock.h"
#include <QMessageBox>
#include <QProgressBar>
#include "QLLTModule.h"
#include "QLLTToolBar.h"
#include "QLLTCreatorDialog.h"
#include "../Misc/LineButton.h"
#include "../Misc/LinePickerDlg.h"
#include "QLLTModule.h"
#include "QLLTSimulation.h"
#include "../src/StoreAssociatedComboBox.h"
#include "../src/GlobalFunctions.h"

QLLTDock::QLLTDock(const QString & title, QMainWindow * parent, Qt::WindowFlags flags, QLLTModule *module)
    : ScrolledDock (title, parent, flags)
{
    setFeatures(QDockWidget::NoDockWidgetFeatures | QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable);
    setAllowedAreas(Qt::LeftDockWidgetArea);
    setVisible(false);

    m_module = module;
    m_shownLLTSimulation = NULL;

    QGroupBox *groupBox = new QGroupBox ("LLT Controls");
    m_contentVBox->addWidget(groupBox);
        QGridLayout *grid = new QGridLayout ();
        groupBox->setLayout(grid);
            m_startSimulationButton = new QPushButton (tr("Start Simulation"));
            connect(m_startSimulationButton, SIGNAL(clicked()), this, SLOT(onStartSimulationButtonClicked()));
            grid->addWidget(m_startSimulationButton, 0, 0, 1, 2);
            m_renameButton = new QPushButton (tr("Rename"));
            connect(m_renameButton, SIGNAL(clicked()), this, SLOT(onRenameButtonClicked()));
            grid->addWidget (m_renameButton, 1, 0);
            m_editCopyButton = new QPushButton (tr("Edit/Copy"));
            connect(m_editCopyButton, SIGNAL(clicked()), this, SLOT(onEditCopyButtonClicked()));
            grid->addWidget (m_editCopyButton, 1, 1);
            m_deleteButton = new QPushButton (tr("Delete"));
            connect(m_deleteButton, SIGNAL(clicked()), this, SLOT(onDeleteButtonClicked()));
            grid->addWidget (m_deleteButton, 2, 0);
            m_newButton = new QPushButton (tr("New"));
            connect(m_newButton, SIGNAL(clicked()), this, SLOT(onNewButtonClicked()));
            grid->addWidget (m_newButton, 2, 1);

    groupBox = new QGroupBox(tr("Progress"));
    m_contentVBox->addWidget(groupBox);
    grid = new QGridLayout ();
            groupBox->setLayout(grid);
            m_progress = new QProgressBar;
            m_progress->setAlignment(Qt::AlignHCenter);
            grid->addWidget(m_progress,0,0);
            stopButton = new QPushButton(tr("STOP"));
            stopButton->setEnabled(false);
            grid->addWidget(stopButton,0,1);
            connect(stopButton, SIGNAL(clicked()), m_module, SLOT(onStopSimulation()));

            continueButton = new QPushButton(tr("CONTINUE"));
            continueButton->setEnabled(false);
            grid->addWidget(continueButton,0,1);
            connect(continueButton, SIGNAL(clicked()), this, SLOT(onContinueSimulationButtonClicked()));
            continueButton->setVisible(false);

            m_sectionBox = new QGroupBox(tr("Section to plot:"));
            m_contentVBox->addWidget(m_sectionBox);
            grid = new QGridLayout ();
            m_sectionBox->setLayout(grid);
            QLabel *label = new QLabel(tr("Section in % of radius:"));
            grid->addWidget(label,0,0);
            m_sectionEdit = new QDoubleSpinBox;
            m_sectionEdit->setMinimum(0);
            m_sectionEdit->setMaximum(1);
            m_sectionEdit->setSingleStep(0.01);
            m_sectionEdit->setDecimals(3);
            grid->addWidget(m_sectionEdit,0,1);
            m_absoluteSection = new QLabel(tr("= x [m]"));
            grid->addWidget(m_absoluteSection,1,0);

            connect(m_sectionEdit,SIGNAL(valueChanged(double)),this,SLOT(onSectionChanged(double)));

    m_stylebox = new QGroupBox (tr("Styles"));
    m_contentVBox->addWidget(m_stylebox);
    grid = new QGridLayout ();
    m_stylebox->setLayout(grid);
            int gridRowCount = 0;
            label = new QLabel (tr("Style: "));
            grid->addWidget(label, gridRowCount, 0);
            m_simulationLineButton = new LineButton ();
            grid->addWidget(m_simulationLineButton, gridRowCount++, 1);
            QHBoxLayout *hBox = new QHBoxLayout ();
            grid->addLayout(hBox, gridRowCount++, 0, 1, 2);
            m_showCheckBox = new QCheckBox (tr("show"));
            hBox->addWidget(m_showCheckBox);
            m_showCurveCheckBox = new QCheckBox (tr("curve"));
            hBox->addWidget(m_showCurveCheckBox);
            m_showPointsCheckBox = new QCheckBox (tr("points"));
            hBox->addWidget(m_showPointsCheckBox);

            connect(m_simulationLineButton, SIGNAL(clicked()), this, SLOT(onLineButtonClicked()));
            connect(m_showCheckBox, SIGNAL(stateChanged(int)), this, SLOT(onShowCheckBoxCanged()));
            connect(m_showCurveCheckBox, SIGNAL(stateChanged(int)), this, SLOT(onShowCurveCheckBoxCanged()));
            connect(m_showPointsCheckBox, SIGNAL(stateChanged(int)), this, SLOT(onShowPointsCheckBoxCanged()));




    m_cutBox = new QGroupBox(tr("Velocity Cut Plane"));
    m_contentVBox->addWidget(m_cutBox);
    QVBoxLayout *vBox = new QVBoxLayout ();
    hBox = new QHBoxLayout;
    grid = new QGridLayout ();

    m_VAWTCutPlaneBox = new LLTCutPlaneComboBox(&g_QLLTCutPlaneStore);
    m_VAWTCutPlaneBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    m_VAWTCutPlaneBox->setMinimumWidth(170);
    m_VAWTCutPlaneBox->setParentBox(m_module->GetQLLTToolBar()->m_LLTVAWTSimulationComboBox);
    vBox->addWidget(m_VAWTCutPlaneBox);

    m_HAWTCutPlaneBox = new LLTCutPlaneComboBox(&g_QLLTCutPlaneStore);
    m_HAWTCutPlaneBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    m_HAWTCutPlaneBox->setMinimumWidth(170);
    m_HAWTCutPlaneBox->setParentBox(m_module->GetQLLTToolBar()->m_LLTHAWTSimulationComboBox);
    vBox->addWidget(m_HAWTCutPlaneBox);

    vBox->addLayout(hBox);
    vBox->addLayout(grid);

    m_cutBox->setLayout(vBox);
    label = new QLabel(tr("Plane Center"));
    hBox->addWidget(label);
    label = new QLabel(tr("X"));
    grid->addWidget(label,0,0);
    m_x_cut = new QDoubleSpinBox;
    grid->addWidget(m_x_cut,0,1);
    label = new QLabel(tr("Y"));
    grid->addWidget(label,1,0);
    m_y_cut = new QDoubleSpinBox;
    grid->addWidget(m_y_cut,1,1);
    label = new QLabel(tr("Z"));
    grid->addWidget(label,2,0);
    m_z_cut = new QDoubleSpinBox;
    grid->addWidget(m_z_cut,2,1);

    label = new QLabel(tr("Plane Rotation"));
    hBox->addWidget(label);
    label = new QLabel(tr("X"));
    grid->addWidget(label,0,2);
    m_x_rot = new QDoubleSpinBox;
    grid->addWidget(m_x_rot,0,3);
    label = new QLabel(tr("Y"));
    grid->addWidget(label,1,2);
    m_y_rot = new QDoubleSpinBox;
    grid->addWidget(m_y_rot,1,3);
    label = new QLabel(tr("Z"));
    grid->addWidget(label,2,2);
    m_z_rot = new QDoubleSpinBox;
    grid->addWidget(m_z_rot,2,3);

    label = new QLabel(tr("Length"));
    grid->addWidget(label,3,0);
    m_length = new QDoubleSpinBox;
    grid->addWidget(m_length,3,1);
    label = new QLabel(tr("Width"));
    grid->addWidget(label,3,2);
    m_width = new QDoubleSpinBox;
    grid->addWidget(m_width,3,3);

    label = new QLabel(tr("X Res"));
    grid->addWidget(label,4,0);
    m_X_res = new QSpinBox;
    grid->addWidget(m_X_res,4,1);
    label = new QLabel(tr("Y Res"));
    grid->addWidget(label,4,2);
    m_Y_res = new QSpinBox;
    grid->addWidget(m_Y_res,4,3);
    m_cutPlaneTime = new QLabel(tr("At Time: [-]"));
    hBox = new QHBoxLayout;
    label = new QLabel(" for all timesteps");
    m_allTimeSteps = new QCheckBox;
    hBox->addWidget(m_cutPlaneTime);
    hBox->addWidget(label);
    hBox->addWidget(m_allTimeSteps);
    vBox->addLayout(hBox);


    hBox = new QHBoxLayout;
    m_addPlane = new QPushButton(tr("New"));
    hBox->addWidget(m_addPlane);
    m_canceldeletePlane = new QPushButton(tr("Delete Single"));
    hBox->addWidget(m_canceldeletePlane);
    vBox->addLayout(hBox);
    hBox = new QHBoxLayout;
    m_deleteAllPlanes = new QPushButton(tr("Delete All"));
    hBox->addWidget(m_deleteAllPlanes);
    m_exportPlane = new QPushButton(tr("Export"));
    hBox->addWidget(m_exportPlane);
    vBox->addLayout(hBox);


    m_x_cut->setDecimals(2);
    m_y_cut->setDecimals(2);
    m_z_cut->setDecimals(2);
    m_x_rot->setDecimals(2);
    m_y_rot->setDecimals(2);
    m_z_rot->setDecimals(2);

    m_x_rot->setMinimum(-180);
    m_y_rot->setMinimum(-180);
    m_z_rot->setMinimum(-180);
    m_x_rot->setMaximum(180);
    m_y_rot->setMaximum(180);
    m_z_rot->setMaximum(180);

    m_x_cut->setMinimum(-10e5);
    m_y_cut->setMinimum(-10e5);
    m_z_cut->setMinimum(-10e5);
    m_x_cut->setMaximum(10e5);
    m_y_cut->setMaximum(10e5);
    m_z_cut->setMaximum(10e5);

    m_x_cut->setSingleStep(1);
    m_y_cut->setSingleStep(1);
    m_z_cut->setSingleStep(1);
    m_x_rot->setSingleStep(5);
    m_y_rot->setSingleStep(5);
    m_z_rot->setSingleStep(5);

    m_X_res->setSingleStep(10);
    m_Y_res->setSingleStep(10);
    m_X_res->setMinimum(1);
    m_Y_res->setMinimum(1);
    m_X_res->setMaximum(10e5);
    m_Y_res->setMaximum(10e5);

    m_width->setSingleStep(1);
    m_length->setSingleStep(1);
    m_width->setMinimum(0);
    m_length->setMinimum(0);
    m_width->setMaximum(10e5);
    m_length->setMaximum(10e5);

    m_X_res->setEnabled(false);
    m_Y_res->setEnabled(false);
    m_x_rot->setEnabled(false);
    m_y_rot->setEnabled(false);
    m_z_rot->setEnabled(false);
    m_x_cut->setEnabled(false);
    m_y_cut->setEnabled(false);
    m_z_cut->setEnabled(false);
    m_width->setEnabled(false);
    m_length->setEnabled(false);
    m_allTimeSteps->setEnabled(false);

    m_X_res->setMaximumSize(60,30);
    m_Y_res->setMaximumSize(60,30);
    m_x_rot->setMaximumSize(60,30);
    m_y_rot->setMaximumSize(60,30);
    m_z_rot->setMaximumSize(60,30);
    m_x_cut->setMaximumSize(60,30);
    m_y_cut->setMaximumSize(60,30);
    m_z_cut->setMaximumSize(60,30);
    m_width->setMaximumSize(60,30);
    m_length->setMaximumSize(60,30);


    connect(m_addPlane, SIGNAL(clicked()), this, SLOT(OnCreateCutPlane()));
    connect(m_canceldeletePlane, SIGNAL(clicked()), m_module, SLOT(OnDeleteCutPlane()));
    connect(m_deleteAllPlanes, SIGNAL(clicked()), m_module, SLOT(OnDeleteAllCutPlanes()));
    connect(m_exportPlane, SIGNAL(clicked()), this, SLOT(OnExportPlane()));

    int checkButtonWidth = 115;

    m_vizbox = new QGroupBox(tr("Turbine Visualization"));
    m_contentVBox->addWidget(m_vizbox);
    grid = new QGridLayout ();
    vBox = new QVBoxLayout;
    m_vizbox->setLayout(vBox);
    vBox->addLayout(grid);
        m_shed = new QPushButton(tr("Shed Vortices"));
        m_shed->setCheckable(true);
        m_shed->setFixedWidth(checkButtonWidth);
        m_shed->setFlat(true);
        grid->addWidget(m_shed,0,0);

        m_trail = new QPushButton(tr("Trailing Vortices"));
        m_trail->setCheckable(true);
        m_trail->setFixedWidth(checkButtonWidth);
        m_trail->setFlat(true);

        grid->addWidget(m_trail,0,1);

        m_nodes = new QPushButton(tr("Vortex Nodes"));
        m_nodes->setCheckable(true);
        m_nodes->setFixedWidth(checkButtonWidth);
        m_nodes->setFlat(true);

        grid->addWidget(m_nodes,1,0);

        QPalette palette = m_nodes->palette();
        palette.setColor(QPalette::Button,QColor(Qt::white));


        m_singleblade = new QPushButton(tr("Single Blade Wake"));
        m_singleblade->setCheckable(true);
        m_singleblade->setFixedWidth(checkButtonWidth);
        m_singleblade->setFlat(true);

        grid->addWidget(m_singleblade,1,1);

        m_showRotor = new QPushButton(tr("Rotor"));
        m_showRotor->setCheckable(true);
        m_showRotor->setFixedWidth(checkButtonWidth);
        m_showRotor->setFlat(true);

        grid->addWidget(m_showRotor,2,0);

        m_showModel = new QPushButton(tr("Turbine Model"));
        m_showModel->setCheckable(true);
        m_showModel->setFixedWidth(checkButtonWidth);
        m_showModel->setFlat(true);

        grid->addWidget(m_showModel,2,1);

        m_showCoordinates = new QPushButton(tr("Coordinate Systems"));
        m_showCoordinates->setCheckable(true);
        m_showCoordinates->setFixedWidth(checkButtonWidth);
        m_showCoordinates->setFlat(true);

        grid->addWidget(m_showCoordinates,3,0);

        m_perspectiveView = new QPushButton(tr("Perspective View"));
        m_perspectiveView->setCheckable(true);
        m_perspectiveView->setFixedWidth(checkButtonWidth);
        m_perspectiveView->setFlat(true);

        grid->addWidget(m_perspectiveView,3,1);

        m_showText = new QPushButton(tr("Show Text"));
        m_showText->setCheckable(true);
        m_showText->setFixedWidth(checkButtonWidth);
        m_showText->setFlat(true);

        grid->addWidget(m_showText,4,0);

        m_showWindfield = new QPushButton(tr("Show WindField"));
        m_showWindfield->setCheckable(true);
        m_showWindfield->setFixedWidth(checkButtonWidth);
        m_showWindfield->setFlat(true);

        grid->addWidget(m_showWindfield,4,1);

        m_colorNodes = new QPushButton(tr("Map Nodes by Age"));
        m_colorNodes->setCheckable(true);
        m_colorNodes->setFixedWidth(checkButtonWidth);
        m_colorNodes->setFlat(true);

        grid->addWidget(m_colorNodes,5,0);

        m_colorLines = new QPushButton(tr("Map Lines by Strength"));
        m_colorLines->setCheckable(true);
        m_colorLines->setFixedWidth(checkButtonWidth);
        m_colorLines->setFlat(true);

        grid->addWidget(m_colorLines,5,1);


        cutVizBox = new QGroupBox(tr("Velocity Cut Plane Visualization"));
        m_contentVBox->addWidget(cutVizBox);
        vBox = new QVBoxLayout;
        cutVizBox->setLayout(vBox);
        grid = new QGridLayout;
        vBox->addLayout(grid);


        m_showCutPlanes = new QPushButton(tr("Show Cut Planes"));
        m_showCutPlanes->setCheckable(true);
        m_showCutPlanes->setFixedWidth(checkButtonWidth);
        m_showCutPlanes->setFlat(true);

        grid->addWidget(m_showCutPlanes,0,0);

        m_showVelVectors = new QPushButton(tr("Show Vectors"));
        m_showVelVectors->setCheckable(true);
        m_showVelVectors->setFixedWidth(checkButtonWidth);
        m_showVelVectors->setFlat(true);

        grid->addWidget(m_showVelVectors,0,1);



        QHBoxLayout *miniHBox = new QHBoxLayout();
        label = new QLabel(tr("Colormap: "));
        miniHBox->addWidget(label);
        m_colorMapButton = new QButtonGroup(miniHBox);
        QRadioButton *radioButton = new QRadioButton ("R/B");
        m_colorMapButton->addButton(radioButton, 0);
        miniHBox->addWidget(radioButton);
        radioButton = new QRadioButton ("Color");
        m_colorMapButton->addButton(radioButton, 1);
        miniHBox->addWidget(radioButton);
        m_colorMapButton->button(0)->setChecked(true);

        QLabel *lab = new QLabel(tr("Range"));
        m_range = new QDoubleSpinBox;
        m_range->setSingleStep(0.01);
        m_range->setMinimum(0.01);
        m_range->setMaximum(1.0);
        m_range->setValue(0.8);
        miniHBox->addWidget(lab);
        miniHBox->addWidget(m_range);

        vBox->addLayout(miniHBox);

        grid = new QGridLayout();
        label = new QLabel(tr("Component: "));
        grid->addWidget(label,0,0);
        m_componentButton = new QButtonGroup(miniHBox);

        radioButton = new QRadioButton ("Total");
        m_componentButton->addButton(radioButton, 0);
        grid->addWidget(radioButton,1,0);

        radioButton = new QRadioButton ("X");
        m_componentButton->addButton(radioButton, 1);
        grid->addWidget(radioButton,0,1);

        radioButton = new QRadioButton ("Y");
        m_componentButton->addButton(radioButton, 2);
        grid->addWidget(radioButton,0,2);

        radioButton = new QRadioButton ("Z");
        m_componentButton->addButton(radioButton, 3);
        grid->addWidget(radioButton,0,3);

        radioButton = new QRadioButton ("Axial");
        m_componentButton->addButton(radioButton, 4);
        grid->addWidget(radioButton,1,1);

        radioButton = new QRadioButton ("Tang.");
        m_componentButton->addButton(radioButton, 5);
        grid->addWidget(radioButton,1,2);

        radioButton = new QRadioButton ("Radial");
        m_componentButton->addButton(radioButton, 6);
        grid->addWidget(radioButton,1,3);

        m_componentButton->button(0)->setChecked(true);
        vBox->addLayout(grid);

        m_showWindfield->setChecked(true);
        m_showVelVectors->setChecked(false);
        m_showCutPlanes->setChecked(true);
        m_nodes->setChecked(true);
        m_trail->setChecked(true);
        m_shed->setChecked(true);
        m_showRotor->setChecked(true);
        m_showCoordinates->setChecked(false);
        m_showModel->setChecked(true);
        m_singleblade->setChecked(false);
        m_perspectiveView->setChecked(false);
        m_showText->setChecked(true);
        m_colorLines->setChecked(false);
        m_colorNodes->setChecked(false);



        connect(m_shed, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_trail, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_nodes, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_colorNodes, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_colorLines, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));

        connect(m_showRotor, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_showText, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));

        connect(m_showCoordinates, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_showModel, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));

        connect(m_singleblade, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_perspectiveView, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));

        connect(m_colorMapButton, SIGNAL(buttonToggled(int,bool)), m_module, SLOT(OnRenderCutPlanes()));
        connect(m_colorMapButton, SIGNAL(buttonToggled(int,bool)), m_module, SLOT(forceReRender()));
        connect(m_componentButton, SIGNAL(buttonToggled(int,bool)), m_module, SLOT(OnComponentChanged()));
        connect(m_componentButton, SIGNAL(buttonToggled(int,bool)), m_module, SLOT(OnRenderCutPlanes()));
        connect(m_componentButton, SIGNAL(buttonToggled(int,bool)), m_module, SLOT(forceReRender()));
        connect(m_range, SIGNAL(valueChanged(double)), m_module, SLOT(OnRenderCutPlanes()));
        connect(m_range, SIGNAL(valueChanged(double)), m_module, SLOT(forceReRender()));


        connect(m_showWindfield, SIGNAL(toggled(bool)), m_module, SLOT(forceReRender()));
        connect(m_showCutPlanes, SIGNAL(toggled(bool)), m_module, SLOT(OnRenderCutPlanes()));
        connect(m_showVelVectors, SIGNAL(toggled(bool)), m_module, SLOT(OnRenderCutPlanes()));






    m_contentVBox->addStretch(10000000);

    addScrolledDock(Qt::LeftDockWidgetArea , parent);

    setShownLLTSimulation(NULL);


    connect(&g_QLLTCutPlaneStore, SIGNAL(objectListChanged(bool)), m_module, SLOT(OnSelChangeCutPlane()));


}

void QLLTDock::onSectionChanged(double section){
    if (m_module->GetQLLTSim()) {
        if (m_module->GetQLLTSim()->getIsVawt()){
            for (int i = 0; i < g_QLLTVAWTSimulationStore.size(); ++i) {
               g_QLLTVAWTSimulationStore.at(i)->setShownBladeSection(section);
            }
        }
        else{
            for (int i = 0; i < g_QLLTHAWTSimulationStore.size(); ++i) {
               g_QLLTHAWTSimulationStore.at(i)->setShownBladeSection(section);
            }
        }
        m_module->updateTimeGraphs();
        if (m_module->GetQLLTSim()->getUsedBlade()->getRotorRadius()*section <= m_module->GetQLLTSim()->getUsedBlade()->m_TPos[0]) m_absoluteSection->setText("= "+QString().number(m_module->GetQLLTSim()->getUsedBlade()->m_TPos[0])+" [m]");
        else m_absoluteSection->setText("= "+QString().number(m_module->GetQLLTSim()->getUsedBlade()->getRotorRadius()*section)+" [m]");
    }
    else{
        m_absoluteSection->setText("= x [m]");
    }
}

void QLLTDock::adjustShowCheckBox() {
    m_showCheckBox->setChecked(m_module->getShownQLLTSimulation()->isShownInGraph());
}

void QLLTDock::OnCreateCutPlane(){

    if (!m_module->GetQLLTSim()) return;
    if (!m_module->GetQLLTSim()->hasStoredData()) return;

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    disconnect(m_canceldeletePlane, SIGNAL(clicked()), m_module, SLOT(OnDeleteCutPlane()));
    connect(m_canceldeletePlane, SIGNAL(clicked()), this, SLOT(OnCancelCutPlane()));
    m_canceldeletePlane->setText(tr("Cancel"));

    m_HAWTCutPlaneBox->setEnabled(false);
    m_VAWTCutPlaneBox->setEnabled(false);

    QString name = m_module->GetQLLTSim()->getName() + " Plane";
    for (int i=0;i<g_QLLTCutPlaneStore.size();i++){
        if (name == g_QLLTCutPlaneStore.at(i)->getName()){
            name = makeNameWithHigherNumber(name);
            i--;
        }
    }
    QLLTCutPlane *plane = new QLLTCutPlane(name, m_module->GetQLLTSim());
    if (m_module->GetQLLTSim()->getStoreWake()){
    plane->m_time = m_module->GetQLLTSim()->getResultsArray().at(0)->at(m_module->GetQLLTSim()->getShownTimeIndex());
    }
    else plane->m_time = m_module->GetQLLTSim()->getResultsArray().at(0)->at(m_module->GetQLLTSim()->getResultsArray().at(0)->size()-1);

    QString strong;
    m_cutPlaneTime->setText("At Time: "+strong.number(plane->m_time,'f',3)+" [s]");

    m_addPlane->setText(tr("Compute"));
    m_addPlane->disconnect();
    connect(m_addPlane, SIGNAL(clicked()), this, SLOT(OnComputeCutPlane()));

    m_X_res->setEnabled(true);
    m_Y_res->setEnabled(true);
    m_x_rot->setEnabled(true);
    m_y_rot->setEnabled(true);
    m_z_rot->setEnabled(true);
    m_x_cut->setEnabled(true);
    m_y_cut->setEnabled(true);
    m_z_cut->setEnabled(true);
    m_width->setEnabled(true);
    m_length->setEnabled(true);
    m_allTimeSteps->setEnabled(true);
    m_allTimeSteps->setChecked(false);

    if (!m_module->GetCutPlane()){
    m_X_res->setValue(50);
    m_Y_res->setValue(50);
    m_width->setValue(m_module->GetQLLTSim()->getUsedBlade()->getRotorRadius()*2.5);
    m_length->setValue(m_module->GetQLLTSim()->getUsedBlade()->getRotorRadius()*2.5);
    m_x_rot->setValue(0);
    m_y_rot->setValue(0);
    m_z_rot->setValue(0);
    m_x_cut->setValue(0);
    m_y_cut->setValue(0);
    m_z_cut->setValue(m_module->GetQLLTSim()->getTowerHeight());
    }

    connect(m_x_cut, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_y_cut, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_z_cut, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_x_rot, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_y_rot, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_z_rot, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_width, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_length, SIGNAL(valueChanged(double)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_X_res, SIGNAL(valueChanged(int)), m_module, SLOT(OnUpdateCutPlane()));
    connect(m_Y_res, SIGNAL(valueChanged(int)), m_module, SLOT(OnUpdateCutPlane()));

    m_module->SetCutPlane(plane);

    m_startSimulationButton->setEnabled(false);
    m_editCopyButton->setEnabled(false);
    m_deleteButton->setEnabled(false);
    m_renameButton->setEnabled(false);
    m_newButton->setEnabled(false);
    continueButton->setEnabled(false);

    m_deleteAllPlanes->setEnabled(false);
    m_exportPlane->setEnabled(false);

    m_module->DisableButtons();

    m_module->OnUpdateCutPlane();


}

void QLLTDock::OnComputeCutPlane(){

    m_addPlane->setText(tr("New"));
    m_addPlane->disconnect();
    connect(m_addPlane, SIGNAL(clicked()), this, SLOT(OnCreateCutPlane()));

    m_HAWTCutPlaneBox->setEnabled(true);
    m_VAWTCutPlaneBox->setEnabled(true);

    m_X_res->setEnabled(false);
    m_Y_res->setEnabled(false);
    m_x_rot->setEnabled(false);
    m_y_rot->setEnabled(false);
    m_z_rot->setEnabled(false);
    m_x_cut->setEnabled(false);
    m_y_cut->setEnabled(false);
    m_z_cut->setEnabled(false);
    m_width->setEnabled(false);
    m_length->setEnabled(false);
    m_allTimeSteps->setEnabled(false);

    m_X_res->disconnect();
    m_Y_res->disconnect();
    m_width->disconnect();
    m_length->disconnect();
    m_x_rot->disconnect();
    m_y_rot->disconnect();
    m_z_rot->disconnect();
    m_x_cut->disconnect();
    m_y_cut->disconnect();
    m_z_cut->disconnect();

    int time;
    QLLTCutPlane *plane;

    if (m_allTimeSteps->isChecked() && m_module->GetQLLTSim()->getStoreWake()){
        QProgressDialog progress("Computing Cut Planes...","Cancel",0,m_module->GetQLLTSim()->getResultsArray().at(0)->size()-1,this);
        progress.setModal(true);
        disconnect(&g_QLLTCutPlaneStore, SIGNAL(objectListChanged(bool)), m_module, SLOT(OnSelChangeCutPlane()));

        plane = m_module->GetCutPlane();
        m_module->SetCutPlane(NULL);
        delete plane;
        for (int i=0;i<m_module->GetQLLTSim()->getResultsArray().at(0)->size();i++){

            QString name = m_module->GetQLLTSim()->getName() + " Plane";
            for (int j=0;j<g_QLLTCutPlaneStore.size();j++){
                if (name == g_QLLTCutPlaneStore.at(j)->getName()){
                    name = makeNameWithHigherNumber(name);
                    j--;
                }
            }
            plane = new QLLTCutPlane(name, m_module->GetQLLTSim());
            plane->m_X_res = m_X_res->value();
            plane->m_Y_res = m_Y_res->value();
            plane->m_width = m_width->value();
            plane->m_length = m_length->value();
            plane->m_X_rot = m_x_rot->value();
            plane->m_Y_rot = m_y_rot->value();
            plane->m_Z_rot = m_z_rot->value();
            plane->m_X = m_x_cut->value();
            plane->m_Y = m_y_cut->value();
            plane->m_Z = m_z_cut->value();
            plane->m_time = m_module->GetQLLTSim()->getResultsArray().at(0)->at(i);
            plane->m_Hub = m_module->GetQLLTSim()->m_savedCoordinateSystems.at(i).at(1);
            plane->m_Axis = m_module->GetQLLTSim()->m_savedCoordinateSystems.at(i).at(0);
            plane->Update();
            m_module->GetQLLTSim()->onComputeCutPlane(plane, i);
            g_QLLTCutPlaneStore.add(plane);

            if (progress.wasCanceled()) break;
            progress.setValue(i);
        }
        connect(&g_QLLTCutPlaneStore, SIGNAL(objectListChanged(bool)), m_module, SLOT(OnSelChangeCutPlane()));
    }
    else{
    if (!m_module->GetQLLTSim()->getStoreWake()) time = 0;
    else time = m_module->GetQLLTSim()->getShownTimeIndex();
    plane = m_module->GetCutPlane();
    plane->m_Hub = m_module->GetQLLTSim()->m_savedCoordinateSystems.at(0).at(1);
    plane->m_Axis = m_module->GetQLLTSim()->m_savedCoordinateSystems.at(0).at(0);
    m_module->GetQLLTSim()->onComputeCutPlane(plane, time);
    g_QLLTCutPlaneStore.add(plane);
    }

    if (m_module->GetQLLTSim()->getIsVawt()){
        m_VAWTCutPlaneBox->setCurrentObject(plane);
    }
    else{
        m_HAWTCutPlaneBox->setCurrentObject(plane);
    }

    m_startSimulationButton->setEnabled(m_module->GetQLLTSim());
    m_editCopyButton->setEnabled(m_module->GetQLLTSim());
    m_deleteButton->setEnabled(m_module->GetQLLTSim());
    m_renameButton->setEnabled(m_module->GetQLLTSim());
    m_newButton->setEnabled(true);
    m_deleteAllPlanes->setEnabled(true);
    m_exportPlane->setEnabled(true);
    if (m_module->GetQLLTSim()){
    continueButton->setEnabled(m_module->GetQLLTSim()->getCanContinue());
    }

    m_module->EnableButtons();

    disconnect(m_canceldeletePlane, SIGNAL(clicked()), this, SLOT(OnCancelCutPlane()));
    connect(m_canceldeletePlane, SIGNAL(clicked()), m_module, SLOT(OnDeleteCutPlane()));
    m_canceldeletePlane->setText(tr("Delete Single"));

    m_module->OnRenderCutPlanes();
}

void QLLTDock::OnExportPlane(){

    if (!m_module->GetCutPlane()) return;

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    QString fileName = QFileDialog::getSaveFileName(g_mainFrame, QString("Export Graph"), g_mainFrame->m_ExportLastDirName, QString("Text File (*.txt);"));
    m_module->GetCutPlane()->exportPlane(fileName);
}

void QLLTDock::OnCancelCutPlane(){

    if (!m_module->GetCutPlane()) return;
    if (m_module->GetCutPlane()->is_computed) return;

    m_HAWTCutPlaneBox->setEnabled(true);
    m_VAWTCutPlaneBox->setEnabled(true);

    delete m_module->GetCutPlane();
    if (m_module->GetQLLTSim()->getIsVawt()) m_module->SetCutPlane(m_VAWTCutPlaneBox->currentObject());
    else m_module->SetCutPlane(m_HAWTCutPlaneBox->currentObject());

    m_X_res->setEnabled(false);
    m_Y_res->setEnabled(false);
    m_x_rot->setEnabled(false);
    m_y_rot->setEnabled(false);
    m_z_rot->setEnabled(false);
    m_x_cut->setEnabled(false);
    m_y_cut->setEnabled(false);
    m_z_cut->setEnabled(false);
    m_width->setEnabled(false);
    m_length->setEnabled(false);
    m_allTimeSteps->setEnabled(false);

    m_X_res->disconnect();
    m_Y_res->disconnect();
    m_width->disconnect();
    m_length->disconnect();
    m_x_rot->disconnect();
    m_y_rot->disconnect();
    m_z_rot->disconnect();
    m_x_cut->disconnect();
    m_y_cut->disconnect();
    m_z_cut->disconnect();


    m_addPlane->setText(tr("New"));
    disconnect(m_addPlane, SIGNAL(clicked()),0,0);
    connect(m_addPlane, SIGNAL(clicked()), this, SLOT(OnCreateCutPlane()));

    m_startSimulationButton->setEnabled(m_module->GetQLLTSim());
    m_editCopyButton->setEnabled(m_module->GetQLLTSim());
    m_deleteButton->setEnabled(m_module->GetQLLTSim());
    m_renameButton->setEnabled(m_module->GetQLLTSim());
    if (m_module->GetQLLTSim()){
    continueButton->setEnabled(m_module->GetQLLTSim()->getCanContinue());
    }
    m_newButton->setEnabled(true);
    m_deleteAllPlanes->setEnabled(true);
    m_exportPlane->setEnabled(true);

    disconnect(m_canceldeletePlane, SIGNAL(clicked()), this, SLOT(OnCancelCutPlane()));
    connect(m_canceldeletePlane, SIGNAL(clicked()), m_module, SLOT(OnDeleteCutPlane()));
    m_canceldeletePlane->setText(tr("Delete Single"));

    m_module->EnableButtons();

    m_module->OnRenderCutPlanes();
}

void QLLTDock::updateProgress(int i){
    m_progress->setValue(i);
    QString cur;
    QTextStream(&cur) << i;
    QString end;
    QTextStream(&end) << m_progress->maximum();
    m_progress->setFormat("Timestep "+cur+" of "+end);
}

void QLLTDock::initView(){

}

void QLLTDock::onStartSimulationButtonClicked(){
    if (!m_module->GetQLLTSim())
		return;
	
    m_module->OnDeleteAllCutPlanes();

    m_module->GetQLLTSim()->setCanContinue(false);

    m_module->ResetTimestepSlider();

    m_QLLTThread =  new QLLTThread ();
    m_QLLTThread->simulation = m_shownLLTSimulation;
    connect(m_QLLTThread, SIGNAL(started()), this, SLOT(onSimStarted()), Qt::QueuedConnection);
    connect(m_QLLTThread, SIGNAL(finished()), this, SLOT(onSimFinished()), Qt::QueuedConnection);
    m_QLLTThread->start();

}

void QLLTDock::onContinueSimulationButtonClicked(){
    if (!m_module->GetQLLTSim())
        return;

    m_QLLTThread =  new QLLTThread ();
    m_QLLTThread->simulation = m_shownLLTSimulation;
    connect(m_QLLTThread, SIGNAL(started()), this, SLOT(onSimStarted()), Qt::QueuedConnection);
    connect(m_QLLTThread, SIGNAL(finished()), this, SLOT(onSimFinished()), Qt::QueuedConnection);
    m_QLLTThread->start();
}

void QLLTDock::onSimStarted(){

    m_progress->setMinimum(0);
    m_progress->setMaximum(m_shownLLTSimulation->getNumberTimesteps());
    m_progress->setTextVisible(true);

    stopButton->setVisible(true);
    stopButton->setEnabled(true);
    continueButton->setVisible(false);

    m_startSimulationButton->setEnabled(false);
    m_editCopyButton->setEnabled(false);
    m_deleteButton->setEnabled(false);
    m_renameButton->setEnabled(false);
    m_newButton->setEnabled(false);

    m_canceldeletePlane->setEnabled(false);
    m_addPlane->setEnabled(false);
    m_exportPlane->setEnabled(false);
    m_deleteAllPlanes->setEnabled(false);


    m_module->DisableButtons();
}

void QLLTDock::onSimFinished(){

    if (!m_module->GetQLLTSim()->getCanContinue() && m_module->GetQLLTSim()->hasStoredData()){
        m_progress->setValue(0);
        m_progress->setFormat(tr("Finished!"));
        stopButton->setEnabled(false);
        stopButton->setVisible(true);
        continueButton->setVisible(false);
        continueButton->setEnabled(false);
    }
    else{
        stopButton->setEnabled(false);
        stopButton->setVisible(false);
        continueButton->setVisible(true);
        continueButton->setEnabled(true);
    }

    m_canceldeletePlane->setEnabled(true);
    m_addPlane->setEnabled(true);
    m_exportPlane->setEnabled(true);
    m_deleteAllPlanes->setEnabled(true);

    m_startSimulationButton->setText(tr("Restart Simulation"));
    m_startSimulationButton->setEnabled(true);
    m_editCopyButton->setEnabled(true);
    m_deleteButton->setEnabled(true);
    m_renameButton->setEnabled(true);
    m_newButton->setEnabled(true);

    m_module->EnableButtons();
    m_module->UpdateDockToolbar();

    m_QLLTThread->deleteLater();

}

void QLLTDock::onReplayStopped(){

    if (!m_module->GetQLLTSim()) return;
    if (!m_module->GetQLLTSim()->getCanContinue() && m_module->GetQLLTSim()->hasStoredData()){
        continueButton->setEnabled(false);
    }
    else{
        continueButton->setEnabled(true);
    }

    m_canceldeletePlane->setEnabled(true);
    m_addPlane->setEnabled(true);
    m_exportPlane->setEnabled(true);
    m_deleteAllPlanes->setEnabled(true);

    m_startSimulationButton->setEnabled(!m_module->GetQLLTSim()->isRunning());
    m_editCopyButton->setEnabled(!m_module->GetQLLTSim()->isRunning());
    m_deleteButton->setEnabled(!m_module->GetQLLTSim()->isRunning());
    m_renameButton->setEnabled(!m_module->GetQLLTSim()->isRunning());
    m_newButton->setEnabled(!m_module->GetQLLTSim()->isRunning());

}

void QLLTDock::onReplayStarted(){

    if (!m_module->GetQLLTSim()) return;

    continueButton->setEnabled(false);
    m_startSimulationButton->setEnabled(false);
    m_editCopyButton->setEnabled(false);
    m_deleteButton->setEnabled(false);
    m_renameButton->setEnabled(false);
    m_newButton->setEnabled(false);

    m_canceldeletePlane->setEnabled(false);
    m_addPlane->setEnabled(false);
    m_exportPlane->setEnabled(false);
    m_deleteAllPlanes->setEnabled(false);

}


void QLLTDock::onNewButtonClicked() {
    /* check first, if all requirements are fullfilled */

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    QString errorMessage ("");
    if (!m_module->m_bisVAWT){
        if (g_rotorStore.size() == 0) {
            errorMessage.append(tr("\n - No HAWT Rotor in Database"));
        }
    }
    if (m_module->m_bisVAWT){
        if (g_verticalRotorStore.size() == 0) {
            errorMessage.append(tr("\n - No VAWT Rotor in Database"));
        }
    }
//    if (g_windFieldStore.size() == 0) {
//        errorMessage.append(tr("\n - No windfield in database"));
//    }
    if (errorMessage != "") {
        QMessageBox::critical(this, tr("Create New Simulation"), QString(tr("The following error(s) occured:\n") + errorMessage), QMessageBox::Ok);
        return;
    }

    QLLTCreatorDialog *creatorDialog = new QLLTCreatorDialog (NULL, m_module, m_module->m_bisVAWT);
    creatorDialog->exec();
    delete creatorDialog;
}

void QLLTDock::setShownLLTSimulation(QLLTSimulation *newSimulation) {
    /* set all labels to correct values */
    m_shownLLTSimulation = newSimulation;
    m_showCheckBox->setEnabled(m_shownLLTSimulation);
    m_showCurveCheckBox->setEnabled(m_shownLLTSimulation);
    m_showPointsCheckBox->setEnabled(m_shownLLTSimulation);
    m_simulationLineButton->setEnabled(m_shownLLTSimulation);
    m_startSimulationButton->setEnabled(m_shownLLTSimulation);
    m_deleteButton->setEnabled(m_shownLLTSimulation);
    m_editCopyButton->setEnabled(m_shownLLTSimulation);
    m_renameButton->setEnabled(m_shownLLTSimulation);

    if (m_shownLLTSimulation){
    m_canceldeletePlane->setEnabled(m_shownLLTSimulation->hasStoredData());
    m_addPlane->setEnabled(m_shownLLTSimulation->hasStoredData());
    m_exportPlane->setEnabled(m_shownLLTSimulation->hasStoredData());
    m_deleteAllPlanes->setEnabled(m_shownLLTSimulation->hasStoredData());
    }
    else{
        m_canceldeletePlane->setEnabled(m_shownLLTSimulation);
        m_addPlane->setEnabled(m_shownLLTSimulation);
        m_exportPlane->setEnabled(m_shownLLTSimulation);
        m_deleteAllPlanes->setEnabled(m_shownLLTSimulation);
    }

    if (m_shownLLTSimulation){
        m_sectionEdit->setValue(m_shownLLTSimulation->getShownBladeSection());
        if (m_shownLLTSimulation->hasStoredData() && m_shownLLTSimulation->getCanContinue()){
            m_startSimulationButton->setText(tr("Restart Simulation"));
            stopButton->setVisible(false);
            stopButton->setEnabled(false);
            continueButton->setVisible(true);
            continueButton->setEnabled(true);
            m_progress->setTextVisible(true);
            m_progress->setMinimum(0);
            m_progress->setMaximum(m_shownLLTSimulation->getNumberTimesteps());
            m_progress->setValue(m_shownLLTSimulation->getCurrentTimeStep());
            QString cur;
            QTextStream(&cur) << m_shownLLTSimulation->getCurrentTimeStep();
            QString end;
            QTextStream(&end) << m_progress->maximum();
            m_progress->setFormat("Timestep "+cur+" of "+end);
        }
        else if (m_shownLLTSimulation->hasStoredData()){
            m_startSimulationButton->setText(tr("Restart Simulation"));
            stopButton->setVisible(true);
            stopButton->setEnabled(false);
            continueButton->setVisible(false);
            continueButton->setEnabled(false);
            m_progress->setTextVisible(true);
            m_progress->setMinimum(0);
            m_progress->setMaximum(m_shownLLTSimulation->getNumberTimesteps());
            m_progress->setValue(m_shownLLTSimulation->getCurrentTimeStep());
            QString cur;
            QTextStream(&cur) << m_shownLLTSimulation->getCurrentTimeStep();
            QString end;
            QTextStream(&end) << m_progress->maximum();
            m_progress->setFormat("Timestep "+cur+" of "+end);      
        }
        else{
            m_startSimulationButton->setText(tr("Start Simulation"));
            stopButton->setVisible(true);
            stopButton->setEnabled(false);
            continueButton->setVisible(false);
            continueButton->setEnabled(false);
            m_progress->setTextVisible(true);
            m_progress->setMinimum(0);
            m_progress->setMaximum(m_shownLLTSimulation->getNumberTimesteps());
            m_progress->setValue(0);
            QString cur;
            QTextStream(&cur) << 0;
            QString end;
            QTextStream(&end) << m_progress->maximum();
            m_progress->setFormat("Timestep "+cur+" of "+end);
        }
        m_simulationLineButton->setPen(m_shownLLTSimulation->getPen());
        m_showCheckBox->setChecked(m_shownLLTSimulation->isShownInGraph());
        m_showPointsCheckBox->setChecked(m_shownLLTSimulation->isDrawPoints());
        m_showCurveCheckBox->setChecked(m_shownLLTSimulation->isDrawCurve());
        if (m_shownLLTSimulation->isRunning()){
            m_startSimulationButton->setEnabled(false);
            m_deleteButton->setEnabled(false);
            m_editCopyButton->setEnabled(false);
            m_renameButton->setEnabled(false);
        }
    }
    else{
        m_progress->setTextVisible(false);
        m_progress->setMinimum(0);
        m_progress->setMaximum(1);
        m_progress->setValue(0);
    }
    onSectionChanged(m_sectionEdit->value());
}

void QLLTDock::onRenameButtonClicked() {

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    if (!m_module->GetQLLTSim()->getIsVawt()) g_QLLTHAWTSimulationStore.rename(m_shownLLTSimulation);
    else g_QLLTVAWTSimulationStore.rename(m_shownLLTSimulation);
}

void QLLTDock::onEditCopyButtonClicked() {

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    QLLTCreatorDialog *creatorDialog = new QLLTCreatorDialog (m_shownLLTSimulation, m_module, m_module->m_bisVAWT);
    creatorDialog->exec();
    delete creatorDialog;
}

void QLLTDock::onDeleteButtonClicked() {

    m_module->OnStopReplay();
    m_module->GetQLLTToolBar()->onStopReplay();

    if (!m_module->GetQLLTSim()->getIsVawt()) g_QLLTHAWTSimulationStore.remove(m_shownLLTSimulation);
    else g_QLLTVAWTSimulationStore.remove(m_shownLLTSimulation);
}

void QLLTDock::onLineButtonClicked() {
    LinePickerDlg linePicker;
    linePicker.InitDialog(m_simulationLineButton->GetStyle(), m_simulationLineButton->GetWidth(), m_simulationLineButton->GetColor());
    if(linePicker.exec() == QDialog::Accepted) 	{
        m_simulationLineButton->SetStyle(linePicker.GetStyle());
        m_simulationLineButton->SetWidth(linePicker.GetWidth());
        m_simulationLineButton->SetColor(linePicker.GetColor());

        m_shownLLTSimulation->setPen(m_simulationLineButton->getPen());
    }
}

void QLLTDock::onShowCheckBoxCanged () {
    m_shownLLTSimulation->setShownInGraph(m_showCheckBox->isChecked());
    m_module->reportGraphChange();
}

void QLLTDock::onShowPointsCheckBoxCanged () {
    m_shownLLTSimulation->setDrawPoints(m_showPointsCheckBox->isChecked());
    m_module->reportGraphChange();
}

void QLLTDock::onShowCurveCheckBoxCanged () {
    m_shownLLTSimulation->setDrawCurve(m_showCurveCheckBox->isChecked());
    m_module->reportGraphChange();
}

void QLLTDock::OnTwoDView(){
    m_vizbox->hide();
    m_cutBox->hide();
    cutVizBox->hide();
    m_stylebox->show();
    m_sectionBox->show();
}

void QLLTDock::OnGLView(){
    m_stylebox->hide();
    m_sectionBox->hide();
    m_vizbox->show();
    m_cutBox->show();
    cutVizBox->show();
}

void QLLTDock::OnHAWTView(){
    m_VAWTCutPlaneBox->hide();
    m_HAWTCutPlaneBox->show();
}

void QLLTDock::OnVAWTView(){
    m_HAWTCutPlaneBox->hide();
    m_VAWTCutPlaneBox->show();
}



